use crate::{
    config::Config,
    error::*,
    git::{self, get_commit_sig, push, tag},
    serverapi::ApiCollection,
};

use futures::{self, stream::FuturesUnordered, StreamExt};
use libpt::log::info;

#[derive(Debug, Clone, PartialEq, Hash)]
pub struct ReleaseContext {
    pub draft: bool,
    pub prerelease: bool,
    pub username: String,
    pub repository: String,
    pub text: String,
    pub tag: String,
    pub commit_sig: String,
}

pub async fn release(cfg: &Config, apis: &mut ApiCollection) -> Result<()> {
    // TODO: Error handling
    let _changelog = crate::changelog::Changelog::build(cfg)?.to_string();
    let mut repo = git::get_repo()?;
    let tag = tag(&mut repo, cfg).await?.name().unwrap().to_string();
    let commit_sig = get_commit_sig(&repo).await?;
    push(cfg).await?; // we assume that we only need to push the current branch to the singular
                      // remote, expecting that the repositories are somehow mirrored
                      // TODO: push to multiple remotes?

    let mut results = FuturesUnordered::new();
    for api in apis.iter_mut() {
        // TODO: check that auth exists
        let specific_rc = ReleaseContext {
            draft: true,
            prerelease: true,
            username: api
                .get_inner()
                .clone()
                .auth
                .expect("no auth but trying to publish")
                .user,
            repository: api.get_inner().repository.clone(),
            text: crate::changelog::Changelog::build(cfg)?.to_string(),
            tag: tag.clone(),
            commit_sig: commit_sig.clone(),
        };
        info!("pushing release for {}", api.get_inner());
        results.push(api.push_release(specific_rc));
    }

    // wait for the release requests to finish
    while let Some(result) = results.next().await {
        if result.is_err() {
            return Err(result.unwrap_err());
        }
    }

    // TODO: check that the release is made
    // TODO: generate artifacts
    // TODO: upload artifacts
    // TODO: upload artifact signatures

    Ok(())
}
